package barry.planmotion;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.RectF;
import android.view.MotionEvent;
import android.view.View;


public class MyView extends View {

	// the sun is placed in the centre of the display
	int iSunXOffset = 0, iSunYOffset = 0;
	
	// how many planets are we showing right now?
	int iPlanetCount = 4; // show this many planets
	
	// the orbits are shown y default. Touch the main part of the screen to toggle them on/off
	boolean bDrawOrbits = true;
	
	// for storing the current 'time', the increment used per screen-draw and the default increment
	double dTime = 0.0, dTimeTick = 0.1, dTimeTickDefault = 0.1;
	
	double dScalar; // scaling factor based on the narrowest extent of the
	                // display and the size of the orbit of the outermost planet
	
	// bitmap handles for the various planets and display controls
	Bitmap bmpSun, bmpLeft, bmpRight, bmpPause, bmpSpeed;
	
	// margin between the controls and the edges of the display
	float fControlGap = 50;
	
	// planet data from here: http://ssd.jpl.nasa.gov/txt/p_elem_t2.txt
	double[] darrayEccentricity = {0.20563661, 0.00676399, 0.01673163, 0.09336511, 0.04853590, 0.05550825, 0.04685740, 0.00895439, 0.24885238};  
	double[] darrayPlanetAU = {0.38709843, 0.72332102, 1.00000018, 1.52371243, 5.20248019, 9.54149883, 19.18797948, 30.06952752, 39.48686035};
	double[] darrayPlanetYear = {0.2409, 0.6132, 1.0, 1.8809, 11.86, 29.0, 84.0, 164.8, 248.0};
	double[] darrayPlanetLongPeri = {77.45771895, 131.76755713, 102.93005885, -23.91744784, 14.27495244, 92.86136063, 172.43404441, 46.68158724, 224.09702598};
	String dArrayPlanetNames[] = {"Mercury","Venus","Earth", "Mars", "Jupiter", "Saturn", "Uranus", "Neptune", "Pluto"};
    Bitmap[] bmparrayBmp = new Bitmap[10];
    
    Paint myPaint = new Paint();

	// rects of floats for displaying controls and orbits
    RectF rectOval, rectTemp, rectLeft, rectPause, rectRight, rectSpeed, rectPlanets;
	
	public MyView(Context context) 
	    {
		super(context);
		
		// set up the bitmaps once, not in the onDraw loop
		bmparrayBmp[0] = BitmapFactory.decodeResource(getResources(), R.drawable.mercury);
		bmparrayBmp[1] = BitmapFactory.decodeResource(getResources(), R.drawable.venus);
		bmparrayBmp[2] = BitmapFactory.decodeResource(getResources(), R.drawable.earth);
		bmparrayBmp[3] = BitmapFactory.decodeResource(getResources(), R.drawable.mars);
		bmparrayBmp[4] = BitmapFactory.decodeResource(getResources(), R.drawable.jupiter);
		bmparrayBmp[5] = BitmapFactory.decodeResource(getResources(), R.drawable.saturn);
		bmparrayBmp[6] = BitmapFactory.decodeResource(getResources(), R.drawable.uranus);
		bmparrayBmp[7] = BitmapFactory.decodeResource(getResources(), R.drawable.neptune);
		bmparrayBmp[8] = BitmapFactory.decodeResource(getResources(), R.drawable.pluto);

		for (int i = 0; i < 9; i++)
			bmparrayBmp[i] = Bitmap.createScaledBitmap (bmparrayBmp[i], 25, 25, true);
		
		bmpSun = BitmapFactory.decodeResource(getResources(), R.drawable.sun);
		bmpSpeed = BitmapFactory.decodeResource(getResources(), R.drawable.speed);
		
		// set up the rects for the display controls
		rectLeft = new RectF (0, 0, 0, 0);
		bmpLeft = BitmapFactory.decodeResource(getResources(), R.drawable.left);
		     rectLeft.top = fControlGap; 
		     rectLeft.bottom = fControlGap + bmpLeft.getHeight();
		     rectLeft.left = fControlGap;
		     rectLeft.right = fControlGap + bmpLeft.getWidth();
		 
		rectPause = new RectF (0, 0, 0, 0);
		bmpPause = BitmapFactory.decodeResource(getResources(), R.drawable.pause);
   	        rectPause.top = fControlGap; 
	        rectPause.bottom = fControlGap + bmpPause.getHeight();
	        rectPause.left = fControlGap + rectLeft.right;
	        rectPause.right = rectPause.left + bmpPause.getWidth();
	 
		rectRight = new RectF (0, 0, 0, 0);
		bmpRight = BitmapFactory.decodeResource(getResources(), R.drawable.right);
	        rectRight.top = fControlGap; 
	        rectRight.bottom = fControlGap + bmpRight.getHeight();
	        rectRight.left = fControlGap + rectPause.right;
	        rectRight.right = rectRight.left + bmpRight.getWidth();
		
	    rectSpeed = new RectF (0, 0, 0, 0);
		bmpSpeed = BitmapFactory.decodeResource(getResources(), R.drawable.speed);
		    rectSpeed.top = fControlGap; 
		    rectSpeed.bottom = fControlGap + bmpSpeed.getHeight();
			
		rectOval = new RectF (0, 0, 0, 0);
		rectTemp = new RectF (0, 0, 0, 0);
		rectPlanets = new RectF (0, 0, 0, 0);
		
	    }

    public boolean onTouchEvent(final MotionEvent event) 
        {
    	float fX = event.getX();
    	float fY = event.getY();
    	
    	// left control, plays orbits backwards
    	if (rectLeft.contains(fX, fY)) 
    		{
    		if (dTimeTick != 0)
    			dTimeTick = 0 - Math.abs(dTimeTick);
    		else
    			dTimeTick = 0 - dTimeTickDefault;
    			
    		return false;
    		}
        // Right control plays orbits forwards
    	else if (rectRight.contains(fX, fY)) 
    		{
    		if (dTimeTick != 0)
        		dTimeTick = Math.abs(dTimeTick);
    		else
    			dTimeTick = dTimeTickDefault;
    			
    		return false;
    		}
        // Pause control stops orbits
    	// WARNING, hold on tight to something because
    	// Stopping the earth's movement is likely to
    	// cause disruption to a lot of stuff
    	// https://www.youtube.com/watch?v=z1iHe42UCCo
    	// 1 hr 14m 56 s into a great old movie!
    	
    	else if (rectPause.contains(fX, fY)) 
		    {
		    dTimeTick = 0;
		    return false;
		    }
    	// change orbit speed, 1x, 3x, 6x, 10x, 15x
    	else if (rectSpeed.contains(fX, fY)) 
	        {
	        float fSpeedFifth = (rectSpeed.right - rectSpeed.left) / 5f;
	        
	        if (fX > (rectSpeed.left + (4 * fSpeedFifth)))
	        	{
	        	dTimeTick = dTimeTickDefault * 15 * Math.signum(dTimeTick);
		        return false;
	        	}
	        else if (fX > (rectSpeed.left + (3 * fSpeedFifth)))
        	    {
        	    dTimeTick = dTimeTickDefault * 10 * Math.signum(dTimeTick);
    	        return false;
        	    }
	        else if (fX > (rectSpeed.left + (2 * fSpeedFifth)))
        	    {
        	    dTimeTick = dTimeTickDefault * 6 * Math.signum(dTimeTick);
	            return false;
    	        }
	        else if (fX > (rectSpeed.left + (1 * fSpeedFifth)))
    	        {
    	        dTimeTick = dTimeTickDefault * 3 * Math.signum(dTimeTick);
	            return false;
    	        }
	        else if (fX > (rectSpeed.left + (0 * fSpeedFifth)))
    	        {
	            dTimeTick = dTimeTickDefault * 1 * Math.signum(dTimeTick);
                return false;
	            }
	        
	        return false;
	        }
    	else if (rectPlanets.contains(fX, fY))
    	    {
    		iPlanetCount = 1 + (int)(fX / (getWidth() / 9));
    		if (iPlanetCount > 9) iPlanetCount = 9;
    	    }
	
    	// touch anywhere else in the screen to toggle the orbit paths
    	if (fY > (rectLeft.bottom * 1.5) && fY < (getHeight() - 125)) 
    		bDrawOrbits = ! bDrawOrbits;
    	
     	return false; // don't need more than one touch event
        }

    @Override
    public void onConfigurationChanged(Configuration newConfig) 
	    {
        super.onConfigurationChanged(newConfig);
        }
	
    @Override
    protected void onDraw(Canvas canvas){
		// offset location of sun. 0, 0 is top left corner of display 
        
        // set the location of the speed control
        rectSpeed.right = getWidth() - fControlGap;
	    rectSpeed.left = rectSpeed.right - bmpSpeed.getWidth();
		
	    // set the area to be checked for the planet selector controls
	    rectPlanets.left = 0;
	    rectPlanets.right = getWidth();
	    rectPlanets.top = getHeight() - 125;
	    rectPlanets.bottom = getHeight();
	    
        // draw the sun at the centre and the controls at the top
        drawSun (canvas, bmpSun, getWidth() / 2, getHeight() / 2);
        drawControl (canvas, bmpLeft, rectLeft.left, rectLeft.top);
        drawControl (canvas, bmpPause, rectPause.left, rectPause.top);
        drawControl (canvas, bmpRight, rectRight.left, rectRight.top);
        drawControl (canvas, bmpSpeed, rectSpeed.left, rectSpeed.top);
        
        myPaint.setColor(Color.WHITE);
    	myPaint.setTextSize(50);
    	canvas.drawText("" + (int)(Math.abs(dTimeTick / dTimeTickDefault)) + "x", 
    			rectSpeed.left, rectSpeed.top + 30, myPaint);

    	// draw planets on bottom row
    	
    	int iW = getWidth();
    	int iW9 = iW / 9;
    	float fX, fY;
    	
    	myPaint.setTextSize(30);
    	
    	for (int i = 0; i < 9; i++)
    	    {
    		fX = (iW / 9 * i) + ((iW9 - bmparrayBmp[i].getWidth()) / 2);
    		fY = getHeight() - 70;
    		
    		canvas.drawBitmap(bmparrayBmp[i], fX, fY, null);
    		canvas.save();
        	    canvas.rotate(-45f, fX, fY);
    		    canvas.drawText(dArrayPlanetNames[i], fX, fY, myPaint);
    		canvas.restore();
    	    }
    	
    	
        // DO PLANETS /////////////////////////////////////////////////////////
        
    	for (int i = 0; i < iPlanetCount; i++)
    	    {
            dScalar = 0.95 * darrayPlanetAU[i] * (1 / darrayPlanetAU[iPlanetCount - 1]);
            drawPlanet (darrayEccentricity[i], dScalar, darrayPlanetYear[i], canvas, myPaint, 
        	    	dArrayPlanetNames[i], bmparrayBmp[i], darrayPlanetLongPeri[i]);
    	    }
    	
	    invalidate();
	    dTime += dTimeTick;
        }   
    
    public double getE (double dTime, double dEccentricity)
        {
    	// passed the degree count in degrees (duh) and the eccentricity value
    	
	    // returns E
    	double dM1, dD, dE0, dE = 0, // return value E = the mean anomaly
    	       dM; // local value of M in radians
    	      
    	dM = Math.toRadians (dTime);
    	
    	int iSign = 1;
    	
    	if (dM > 0) iSign = 1; else iSign = -1;
    	
    	dM = Math.abs(dM) / (2 * Math.PI); // Meeus, p 206, line 110
    	dM = (dM - (long)dM) * (2 * Math.PI) * iSign; // line 120
    	if (dM < 0)
    		dM = dM + (2 * Math.PI); // line 130
    	iSign = 1;
    	if (dM > Math.PI) iSign = -1; // line 150
    	if (dM > Math.PI) dM = 2 * Math.PI - dM; // line 160
    	
    	dE0 = Math.PI / 2; // line 170
    	dD = Math.PI / 4; // line 170
    	
        for (int i = 0; i < 33; i++) // line 180 
            {
        	dM1 = dE0 - dEccentricity * Math.sin(dE0); // line 190
        	dE0 = dE0 + dD * Math.signum((float)(dM - dM1));
        	dD = dD / 2; 
            }
        
        dE = dE0 * iSign;
        
        return dE;
        }
    
    public double getRfromE (double dE, double dEccentricty, double dScalar)
        {
    	return Math.min(getWidth(), getHeight()) / 2 * dScalar * (1 - (dEccentricty * Math.cos(dE)));
        }
    
    public int getYOffset (double dPlanet1_Eccentricity)
        {
        // can use this if displaying only one orbit and which is
    	// highly eccentric. It returns how far down the screen 
    	// you need to place the focus to ensure the whole orbit is
    	// visible
    	
    	return (int) (
		           getRfromE (Math.PI, dPlanet1_Eccentricity, dScalar)
		           +
		           (
				     getHeight() - (
				    		         getRfromE (Math.PI, dPlanet1_Eccentricity, dScalar) + 
				    		         getRfromE (   (double) 0, dPlanet1_Eccentricity, dScalar)   
				    		       )
	               ) / 2 );
        }
    
    public void drawSun (Canvas canvas, Bitmap bmp, float fX, float fY)
        {
    	int iBitmapHeight = bmp.getHeight();
		      
	    canvas.drawBitmap(bmp, (getWidth() / 2) - (iBitmapHeight / 2), 
	    		               (getHeight() / 2) - (iBitmapHeight / 2), null);
	    }
    
    public void drawControl (Canvas canvas, Bitmap bmp, float fX, float fY)
        {
	    canvas.drawBitmap(bmp, fX, fY, null);
        }


    public void drawPlanet (double dEccentricity, double dScalar, double dYear, Canvas canvas, Paint paint, 
    		String sName, Bitmap bmp, double dLongPeri)
        {
        double dE, dr, dv, dSatX, dSatY, dSatXCorrected, dSatYCorrected;
        float fX, fY;
        int iSunXOffset = getWidth() / 2;
        int iSunYOffset = getHeight() / 2;
        
        // get the value of E from the angle travelled in this 'tick'
	    
	    dE = getE (dTime * (1 / dYear), dEccentricity);
	    
	    // get r: the length of 'radius' vector
	    
	    dr = getRfromE (dE, dEccentricity, dScalar);
	    
	    // calculate v - the true anomaly
	    
	    dv = 2 * Math.atan (
	    		Math.sqrt((1 + dEccentricity) / (1 - dEccentricity))
	    		*
	    		Math.tan(dE / 2)
	    		); 
	    
	    // get X and Y coords based on the origin
	    
	    dSatX = dr / Math.sin(Math.PI / 2) * Math.sin(dv);
	    dSatY = Math.sin((Math.PI / 2) - dv) * (dSatX / Math.sin(dv));
	    
	    // now correct for Longitude of Perihelion for this planet
	    
	    dSatXCorrected = dSatX * (float)Math.cos (Math.toRadians(dLongPeri)) - dSatY * (float)Math.sin(Math.toRadians(dLongPeri));
	    dSatYCorrected = dSatX * (float)Math.sin (Math.toRadians(dLongPeri)) + dSatY * (float)Math.cos(Math.toRadians(dLongPeri));
	    
	    // offset the origin to nearer the centre of the display
	    
	    fX = (float)dSatXCorrected + (float)iSunXOffset;
	    fY = (float)dSatYCorrected + (float)iSunYOffset;
	    
	    if (bDrawOrbits)
        {
            paint.setColor(Color.WHITE);
            paint.setStyle(Paint.Style.STROKE);
            paint.setAntiAlias(true);
        
            // get the size of the rect which encloses the elliptical orbit
        
            dE = getE (0.0, dEccentricity);
            dr = getRfromE (dE, dEccentricity, dScalar);
            rectOval.bottom = (float)dr;
            dE = getE (180.0, dEccentricity);
            dr = getRfromE (dE, dEccentricity, dScalar);
            rectOval.top = (float)(0 - dr);
        
            // calculate minor axis from major axis and eccentricity
            // http://www.1728.org/ellipse.htm
            double dMajor = rectOval.bottom - rectOval.top;
            double dMinor = Math.sqrt(1 - (dEccentricity * dEccentricity)) * dMajor;
        
            rectOval.left = 0 - (float)(dMinor / 2);
            rectOval.right = (float)(dMinor / 2);

            rectOval.left += (float)iSunXOffset;
            rectOval.right += (float)iSunXOffset;
            rectOval.top += (float)iSunYOffset;
            rectOval.bottom += (float)iSunYOffset;
       
            // now correct for Longitude of Perihelion for this orbit's path

            canvas.save();
            	canvas.rotate((float)dLongPeri, (float)iSunXOffset, (float)iSunYOffset);
            	canvas.drawOval(rectOval, paint);
            canvas.restore();
            }
	    
	    
	    int iBitmapHeight = bmp.getHeight();
		
	    canvas.drawBitmap(bmp, fX - (iBitmapHeight / 2), fY - (iBitmapHeight / 2), null);

	    // draw planet label
	    
		myPaint.setColor(Color.WHITE);
		paint.setTextSize(30);
		canvas.drawText(sName, fX+20, fY-20, paint);
        }
    }


